#version 150

uniform sampler2D u_texture;

in vec3 io_eyeVec;
in vec3 io_lightVec;
in vec3 io_normal;
in vec2 io_uv;

out vec4 o_color;

float stepmix(float edge0, float edge1, float E, float x)
{
    float T = clamp(0.5 * (x - edge0 + E) / E, 0.0, 1.0);
    return mix(edge0, edge1, T);
}

void main() {

   vec4 texColor = texture(u_texture, io_uv);
   if (texColor.w < 0.05)
      discard;

   vec3 N = normalize(io_normal);
   vec3 L = normalize(io_lightVec);
   vec3 V = normalize(io_eyeVec);
   vec3 R = normalize(reflect(-L, N));

   vec4 mata = vec4(0.4, 0.4, 0.4, 1.0);
   vec4 matd = vec4(0.8, 0.8, 0.8, 1.0);
   vec4 mats = vec4(1.1, 1.1, 1.1, 1.0);
   float shine = 500.0;

   vec4 ambient = mata * texColor;

   float df = clamp(dot(N, L), 0.0, 1.0);
   float sf = pow(clamp(dot(R, V), 0.0, 1.0), shine);

   const float A = 0.1;
   const float B = 0.3;
   const float C = 0.6;
   const float D = 1.0;
   float E = fwidth(df);

    if      (df > A - E && df < A + E) df = stepmix(A, B, E, df);
    else if (df > B - E && df < B + E) df = stepmix(B, C, E, df);
    else if (df > C - E && df < C + E) df = stepmix(C, D, E, df);
    else if (df < A) df = 0.0;
    else if (df < B) df = B;
    else if (df < C) df = C;
    else df = D;

    E = 0.2;//fwidth(sf);

    if (sf > 0.5 - E && sf < 0.5 + E)
        sf = smoothstep(0.5 - E, 0.5 + E, sf);
    else
        sf = step(0.5, sf);

   vec4 diffuse = matd * texColor * df;
   vec4 specular = mats * texColor * sf;

   o_color = ambient + diffuse + specular;
}
