#version 150

uniform float u_enabled;
uniform vec3 u_color;
uniform sampler2D u_texture;

in vec2 io_uv;
in vec3 io_normal;
in vec3 io_eye;
in vec3 io_pos;

out vec4 o_color;

void main() {

   vec4 color = texture(u_texture, io_uv);

   if (io_pos.y < u_enabled)
      color = color * vec4(u_color, 1.0);
   else
      color = color * vec4(0.5 ,0.5, 0.5, 1.0);

   vec4 final_color = color * 0.15;
   vec3 light = vec3(0.3, 0.0, 1.0);
   float shininess = 20.0;

   vec3 N = normalize(io_normal);
   vec3 L = normalize(light);
   vec3 U = vec3(0, 0, 1);

   float lambertTerm = dot(N, L);
   
   if (lambertTerm > 0)
   {
      final_color += color * lambertTerm;

	  vec3 E = normalize(io_eye);
	  vec3 R = reflect(-L, N);

	  float specular = pow(clamp(dot(R, E), 0.0, 1.0), shininess);
	  final_color += color * specular;
	}

	o_color = final_color;  
}
